// Includes I didn't make myself
#include "SDL/SDL.h"
#include "SDL/SDL_ttf.h"
#include <string>

#ifndef BUTTON_H
#define BUTTON_H

class cButton {
private:
    SDL_Rect propsStandard, propsHeader, propsCurrent;    // propsStandard is the default position of the button.
                                            // propsHeader is the position of the button when it is a header.
                                            // propsCurrent is used when the button is moving to a new position.
    std::string caption;                    // Contains the caption.
    int targetState, currentState;          // Contains the state which it is currently in: fading out, fading
                                            // in, moving to the header position, etc.
    int visibleState;
    int buttonState;                        // Contains the state which the button is currently in: neutral,
                                            // mouse hovering over, clicked.
    int startTimeM;                          // When the button started animating
    int startTimeV;                         // For faing in/out
    SDL_Surface* captionSurface;            // Contains the surface with the button text.
    SDL_Surface* captionSurfaceH;           // Contains the surface with the button text when the mouse hovers over.
    SDL_Surface* fadeSurface;               // Surface used for fading in and out.

public:
    cButton(std::string caption, int x, int y, bool align, TTF_Font* font, int visibleStateF, int position); // Constructor, sets all the custom variables and does a lot
                                            // in advance (like calculating header position, generating button
                                            // text, that kinda stuff).
    ~cButton();                             // Destroys EVERYTHING! MWUHAHAHA.

    int reset();                            // Resets the button: makes it fade in or move in from header position.
                                            // And sets the state to neutral.
    int makeHeader();                       // Sets the state to header.
    int makeStandard();
    int fadeIn();
    int fadeOut();                          // Sets the state to faded out.
    int logic(int mX, int mY, Uint8 mouseState); // Handles input,fading it in/out and moving it
    int render();                           // renders
                                            // to/from the header position.
    int getState();                         // Returns the static state or of applicable the dynamic state.
                                            // Static state is for instance at header positon, or invisible.
                                            // Dynamic state means for instance fading out, or moving from header
                                            // position to neutral position.
    int getVisible();
    int getButtonState();                     // Returns the button state (clicked/hovering/neutral)
    int getProps();                         // Returns x/y depending which state is given (header or menu state)
                                            // and always returns the same width and height.
    std::string getText();                  // Returns the button caption.
} ;

// Button Constants
const int B_NULL = 0;
const int B_NEUTRAL = 1;
const int B_HOVER = 2;
const int B_CLICK = 3;

const int B_FADINGOUT = 4;
const int B_FADINGIN = 5;
const int B_TOHEADER = 6;
const int B_TOSTANDARD = 7;

const int B_HEADER = 8;
const int B_STANDARD = 9;
const int B_INVISIBLE = 10;
const int B_VISIBLE = 11;

const float B_MS = 500; // The amount of milliseconds the animation is going to take

#endif
