
#include "cMazeCell.h"
#include "cMazeField.h"
#include "constants.h"
#include "functions.h"
#include "gamestates.h"
#include "globals.h"

int gsMaze::logic() {
    if (bBack->logic(mX, mY, mouseState) == B_CLICK) {
        gameMan->setNextState(STATE_MENU);
    }

    speed->logic(mX, mY, mouseState);

    path->logic(mX, mY, mouseState);

    int mazeState = maze->logic(speed->getSelected(), path->getSelected());

    int which;
    SDL_Rect waitBox;
    waitBox.y = 25;
    waitBox.w = 5;
    waitBox.h = 5;

    if (mazeState == MAZE_MGNR) {
        if (speed->getSelected() == 0) { // pauze

        } else {
            which = startFrame / 800;
            which = which % 3;
            waitBox.x = 100 + which *(5 + 10);
            SDL_FillRect(screen, &waitBox, 0);
        }
    } else if (mazeState == MAZE_PATH) {
        if (speed->getSelected() == 0) { // pauze

        } else {
            which = startFrame / 800;
            which = which % 3;
            waitBox.x = 100 + which *(5 + 10);
            SDL_FillRect(screen, &waitBox, 0);
        }
    } else if (mazeState == MAZE_DONE) {

    }

    return 0;
}

int gsMaze::render() {
    SDL_FillRect(screen, NULL, SDL_MapRGB(screen->format, 255, 255, 255));

    applySurface(tagSpeed, screen, 310, 0);
    applySurface(tagPause, screen, 200, 22);
    applySurface(tagFast, screen, 450, 22);
    applySurface(tagPath, screen, 151, 0);

    bBack->render();

    speed->render();

    path->render();

    maze->render(path->getSelected());

    SDL_Flip(screen);

    return 0;
}

gsMaze::gsMaze() {
    bBack = new cButton("Terug", 10, 5, false, fNormal, B_VISIBLE, B_STANDARD);

    tagPause = TTF_RenderText_Shaded(fTag, "Pause", cBlack, cWhite);
    tagFast = TTF_RenderText_Shaded(fTag, "Snel", cBlack, cWhite);
    tagSpeed = TTF_RenderText_Shaded(fTag, "Snelheid", cBlack, cWhite);
    tagPath = TTF_RenderText_Shaded(fTag, "Pad", cBlack, cWhite);

    speed = new cCheckboxes(260, 25, 21, true, MAXSPEED, 20, 2);
    path = new cCheckboxes(159, 25, 0, false, 1, 20, 2);

    int mazeSize;
    switch (gameMan->params[PARAM_SIZE]) {
        case 1:
            // maze = new cBinary(5);
            mazeSize = 5;
            break;
        case 2:
            // maze = new cBinary(20);
            mazeSize = 25;
            break;
        case 3:
            // maze = new cBinary(50);
            mazeSize = 50;
            break;
        default:
            // maze = new cBinary(50);
            mazeSize = 50;
            break;
    }

    switch (gameMan->params[PARAM_ALGO]) {
        case 1:
            maze = new cMG(mazeSize);
            break;
        case 2:
            maze = new cHuntKill(mazeSize);
            break;
        case 3:
            maze = new cBinary(mazeSize);
            break;
        case 4:
            maze = new cPrims(mazeSize);
            break;
    }

    switch (gameMan->params[PARAM_SPD]) {
        case 1:
            speed->setSelected(2);
            break;
        case 2:
            speed->setSelected(8);
            break;
        case 3:
            speed->setSelected(9);
            break;
    }
}

gsMaze::~gsMaze() {
    delete bBack;
    delete maze;
    delete speed;

    SDL_FreeSurface(tagPause);
    SDL_FreeSurface(tagFast);
    SDL_FreeSurface(tagSpeed);
}
