#include <SDL/SDL.h>

#include "cMaze.h"
#include "cMazeField.h"
#include "functions.h"
#include "globals.h"
#include "mtwist.h"

int cHuntKill::generateStep() {
    int pM = 0; // pickedMove
    std::vector< std::vector<int> > neighbours;
    std::vector<int> tVector;
    bool stepped = false;

    int t = 0;

    while (!stepped) {
        switch (curState) {
            case HK_HUNT:
                if (curRow >= maze->gSize()) {
                    mazeState = MAZE_DONE;
                    maze->unuse();
                } else {
                    maze->unuse();
                    maze->useRow(curRow);
                    t = maze->rowHasUnvisited(curRow);
                    if (t > -1) { // unvisited in row
                        curX = t;
                        curY = curRow;
                        curState = HK_KILL;
                    } else {  // no unvisited
                        curRow++;
                    }
                }

                stepped = true;
                break;
            case HK_KILL:
                if (maze->hasUnvisitedN(curX, curY)) { // moves available
                    if (neighbours.size() > 0) {
                        for (int i = 0; i < neighbours.size(); i++) {
                            neighbours[i].clear();
                        }
                        neighbours.clear();
                    }

                    // above. pFX = 0 || < size
                    if (inRange(curX, 0, maze->gSize()) && inRange(curY - 1, 0, maze->gSize())) {
                        if (maze->isVisited(curX, curY - 1)  == false) {
                            tVector.clear();
                            tVector.push_back(curX);
                            tVector.push_back(curY - 1);
                            neighbours.push_back(tVector);
                        }
                    }

                    // under
                    if (inRange(curX, 0, maze->gSize()) && inRange(curY + 1, 0, maze->gSize())) {
                        if (maze->isVisited(curX, curY + 1)  == false) {
                            tVector.clear();
                            tVector.push_back(curX);
                            tVector.push_back(curY + 1);
                            neighbours.push_back(tVector);
                        }
                    }

                    // left
                    if (inRange(curX - 1, 0, maze->gSize()) && inRange(curY, 0, maze->gSize())) {
                        if (maze->isVisited(curX - 1, curY)  == false) {
                            tVector.clear();
                            tVector.push_back(curX - 1);
                            tVector.push_back(curY);
                            neighbours.push_back(tVector);
                        }
                    }

                    //right
                    if (inRange(curX + 1, 0, maze->gSize()) && inRange(curY, 0, maze->gSize())) {
                        if (maze->isVisited(curX + 1, curY) == false) {
                            tVector.clear();
                            tVector.push_back(curX + 1);
                            tVector.push_back(curY);
                            neighbours.push_back(tVector);
                        }
                    }

                    pM = mt_lrand() % neighbours.size();

                    maze->connect(curX, curY, neighbours[pM][0], neighbours[pM][1], C_NOWALL);

                    curX = neighbours[pM][0];
                    curY = neighbours[pM][1];

                    maze->unuse();
                    maze->use(curX, curY);
                    maze->visit(curX, curY);

                    stepped = true;
                } else {
                    // LET'S HUNT
                    curState = HK_KILL;
                    curRow = 0;
                }
                break;
        }
    }

    tVector.clear();
    for (int i = 0; i < neighbours.size(); i++) {
        neighbours[i].clear();
    }
    neighbours.clear();

    return 0;
}

cHuntKill::cHuntKill(int size) {
    lastFrame = 0;
    mazeState = MAZE_MGNR;
    maze = new cMazeField(size);
    surfEnlarged = SDL_CreateRGBSurface(SDL_SWSURFACE, 500, 500, screen->format->BitsPerPixel, screen->format->Rmask, screen->format->Gmask, screen->format->Bmask, screen->format->Amask);

    curX = 0;
    curY = 0;
    maze->visit(curX, curY);
    curState = HK_KILL;
}

cHuntKill::~cHuntKill() {
    delete maze;

    SDL_FreeSurface(surfEnlarged);
}
