#include <SDL/SDL.h>

#include "cMazeCell.h"
#include "cMazeField.h"
#include "globals.h"

int cMazeCell::render(SDL_Surface* dst, cMazeField* container) {
    SDL_Rect cell; // main cell
    SDL_Rect hor; // Horizontal wall
    SDL_Rect ver; // Vertical wall
    SDL_Rect cor; // Corner

    int pX = x * C_SIZE;
    int pY = y * C_SIZE;

    Uint32 cellClr;
    Uint32 c32Black = SDL_MapRGB(dst->format, 0, 0, 0);

    if (current) {
        cellClr = SDL_MapRGB(dst->format, 56, 51, 189);
    } else if (visitedD) {
        cellClr = SDL_MapRGB(dst->format, 255, 179, 209);
    } else if (visited) {
        cellClr = SDL_MapRGB(dst->format, 255, 255, 255);
    } else {
        cellClr = SDL_MapRGB(dst->format, 128, 128, 128);
    }

    cell.w = C_SIZE;
    cell.h = C_SIZE;
    cell.x = pX;
    cell.y = pY;
    SDL_FillRect(dst, &cell, cellClr);

    hor.w = C_SIZE - 2 * C_BORDER;
    hor.h = C_BORDER;
    ver.w = C_BORDER;
    ver.h = C_SIZE - 2 * C_BORDER;
    cor.w = C_BORDER;
    cor.h = C_BORDER;

    // Four corners first
    for (int ix = 0; ix < 2; ix++) {
        for (int iy = 0; iy < 2; iy++) {
            cor.x = pX + ix * (C_SIZE - C_BORDER);
            cor.y = pY + iy * (C_SIZE - C_BORDER);
            SDL_FillRect(dst, &cor, c32Black);
        }
    }

    // Top & left second
    hor.x = pX + C_BORDER;
    hor.y = pY;

    if ((top == C_WALL || y == 0) && !(x == 0 && y == 0)) {
        SDL_FillRect(dst, &hor, c32Black);
    }

    ver.x = pX;
    ver.y = pY + C_BORDER;

    if (left == C_WALL || x == 0) {
        SDL_FillRect(dst, &ver, c32Black);
    }

    // Right & bottom rely on cMazeField
    if (!(x == container->gSize() - 1 && y == container->gSize() - 1))
    if (x == container->gSize() - 1) { // HIER
        ver.x = pX + C_SIZE - C_BORDER;
        SDL_FillRect(dst, &ver, c32Black);
    } else {
        if (container->gWall(x+1, y, CELL_LEFT) == C_WALL) {
            ver.x = pX + C_SIZE - C_BORDER;
            SDL_FillRect(dst, &ver, c32Black);
        }
    }

    if (y == container->gSize() - 1) {
        hor.y = pY + C_SIZE - C_BORDER;
        SDL_FillRect(dst, &hor, c32Black);
    } else {
        if (container->gWall(x, y+1, CELL_TOP) == C_WALL) {
            hor.y = pY + C_SIZE - C_BORDER;
            SDL_FillRect(dst, &hor, c32Black);
        }
    }

    return 0;
}

int cMazeCell::use() {
    current = true;

    return 0;
}

int cMazeCell::unuse() {
    current = false;

    return 0;
}

int cMazeCell::sTop(bool p) {
    top = p;
    return 0;
}

int cMazeCell::sLeft(bool p) {
    left = p;
    return 0;
}

int cMazeCell::visit() {
    visited = true;
    return 0;
}

int cMazeCell::visitD() {
    visitedD = true;
    return 0;
}

bool cMazeCell::gTop() {
    return top;
}

bool cMazeCell::gLeft() {
    return left;
}

bool cMazeCell::gVisit() {
    return visited;
}

bool cMazeCell::gVisitD() {
    return visitedD;
}

cMazeCell::cMazeCell(int x, int y) {
    this->x = x;
    this->y = y;

    top = C_WALL;
    left = C_WALL;
    visited = false;
    visitedD = false;
}

cMazeCell::~cMazeCell() {
    // No resource management needed
}
