#include "mtwist.h"
#include <vector>

#include "cMaze.h"
#include "cMazeField.h"
#include "functions.h"
#include "globals.h"

int cPrims::generateStep() {
    // zijn er nog fronts?
    // Kies random front
    // check of deze nog buren heeft
    //      zo ja:  Voeg willekeurige buur aan front toe

    //      zo nee: haal deze buur uit de lijst, kies nieuw front

    bool stepped = false;
    int pF = 0; // pickedFront
    std::vector< std::vector<int> > neighbours;
    std::vector<int> tVector;
    int pFX = 0, pFY = 0;
    int pNC = 0;
    int pNCX = 0, pNCY = 0;

    while (!stepped) {
        if (fronts.size() == 0) { // No more cells on the list so maze is done!
            mazeState = MAZE_DONE;
            stepped = true;
            maze->unuse();
        } else { // Still fronts left! Maze is not done. p = picked
            pF = mt_lrand() % fronts.size();
            if (maze->hasUnvisitedN(fronts[pF][0], fronts[pF][1]) == true) { // This front has neighbours
                pFX = fronts[pF][0];
                pFY = fronts[pF][1];

                if (neighbours.size() > 0) {
                    for (int i = 0; i < neighbours.size(); i++) {
                        neighbours[i].clear();
                    }
                    neighbours.clear();
                }

                // above. pFX = 0 || < size
                if (inRange(pFX, 0, maze->gSize()) && inRange(pFY - 1, 0, maze->gSize())) {
                    if (maze->isVisited(pFX, pFY - 1)  == false) {
                        tVector.clear();
                        tVector.push_back(pFX);
                        tVector.push_back(pFY - 1);
                        neighbours.push_back(tVector);
                    }
                }

                // under
                if (inRange(pFX, 0, maze->gSize()) && inRange(pFY + 1, 0, maze->gSize())) {
                    if (maze->isVisited(pFX, pFY + 1)  == false) {
                        tVector.clear();
                        tVector.push_back(pFX);
                        tVector.push_back(pFY + 1);
                        neighbours.push_back(tVector);
                    }
                }

                // left
                if (inRange(pFX - 1, 0, maze->gSize()) && inRange(pFY, 0, maze->gSize())) {
                    if (maze->isVisited(pFX - 1, pFY)  == false) {
                        tVector.clear();
                        tVector.push_back(pFX - 1);
                        tVector.push_back(pFY);
                        neighbours.push_back(tVector);
                    }
                }

                //right
                if (inRange(pFX + 1, 0, maze->gSize()) && inRange(pFY, 0, maze->gSize())) {
                    if (maze->isVisited(pFX + 1, pFY) == false) {
                        tVector.clear();
                        tVector.push_back(pFX + 1);
                        tVector.push_back(pFY);
                        neighbours.push_back(tVector);
                    }
                }

                pNC = mt_lrand() % neighbours.size();

                pNCX = neighbours[pNC][0];
                pNCY = neighbours[pNC][1];

                std::vector<int> newFront;
                newFront.push_back(pNCX);
                newFront.push_back(pNCY);
                fronts.push_back(newFront);

                /*
                if (pNCX > pFX) {
                    maze->sWall(pNCX, pNCY, CELL_LEFT, C_NOWALL);
                } else if (pNCX < pFX) {
                    maze->sWall(pNCX, pNCY, CELL_RIGHT, C_NOWALL);
                } else if (pNCY > pFY) {
                    maze->sWall(pNCX, pNCY, CELL_TOP, C_NOWALL);
                } else if (pNCY < pFY) {
                    maze->sWall(pNCX, pNCY, CELL_BOTTOM, C_NOWALL);
                }
                */

                maze->connect(pNCX, pNCY, pFX, pFY, C_NOWALL);

                maze->visit(neighbours[pNC][0], neighbours[pNC][1]);
                maze->unuse();
                maze->use(neighbours[pNC][0], neighbours[pNC][1]);

                stepped = true;

                // check if actual cell still has neighbours
                if (maze->hasUnvisitedN(fronts[pF][0], fronts[pF][1]) == false) {
                    // delete cell from list
                    fronts.erase(fronts.begin() + pF);
                }
            } else { // This front doesn't have neighbours
                // delete from list
                fronts.erase(fronts.begin() + pF);
            }
        }
    }

    tVector.clear();
    for (int i = 0; i < neighbours.size(); i++) {
        neighbours[i].clear();
    }
    neighbours.clear();

    return 0;
}

cPrims::cPrims(int size) {
    lastFrame = 0;
    mazeState = MAZE_MGNR;

    maze = new cMazeField(size);

    std::vector<int> newFront;
    newFront.push_back(0);
    newFront.push_back(0);
    fronts.push_back(newFront);
    newFront.clear();

    maze->visit(0, 0);

    surfEnlarged = SDL_CreateRGBSurface(SDL_SWSURFACE, 500, 500, screen->format->BitsPerPixel, screen->format->Rmask, screen->format->Gmask, screen->format->Bmask, screen->format->Amask);
}

cPrims::~cPrims() {
    delete maze;

    if (fronts.size() > 0) {
        for (int i = 0; i < fronts.size(); i++) {
            fronts[i].clear();
        }

        fronts.clear();
    }

    SDL_FreeSurface(surfEnlarged);
}
