#include <SDL/SDL.h>
#include <SDL/SDL_ttf.h>

#include "button.h"
#include "constants.h"
#include "functions.h"
#include "gamestates.h"
#include "globals.h"

int gsHowTo::logic() {
    bool newM = false;

    if (bBack->logic(mX, mY, mouseState) == B_CLICK) {
        gameMan->setNextState(STATE_MENU);
    }

    int tempo = 6;
    if (mode == HT_DSTR) {tempo = 8;}
    prevMState = mState;
    mState = ex->logic(tempo, true);

    if (prevMState == MAZE_PATH && mState == MAZE_DONE) {
        exFinished = SDL_GetTicks(); // Start timer
    }

    if (exFinished > 0) { // Is timer running
        if (SDL_GetTicks() - exFinished >= 5500) {
            newM = true;
            exFinished = 0;
            prevMState = MAZE_MGNR;
            mState = MAZE_MGNR;
        }
    }

    if (mode != HT_MG) {
        if (bMuurgroeier->logic(mX, mY, mouseState) == B_CLICK) {
            mode = HT_MG;
            bMuurgroeier->setState(B_HOVER);

            delete ex;
            ex = new cMG(10);

            exFinished = 0;
        }
    } else if (mode == HT_MG && newM) {
        delete ex;
        ex = new cMG(10);
    }

    if (mode != HT_HK) {
        if (bHuntKill->logic(mX, mY, mouseState) == B_CLICK)  {
            mode = HT_HK;
            bHuntKill->setState(B_HOVER);

            delete ex;
            ex = new cHuntKill(10);

            exFinished = 0;
        }
    } else if (mode == HT_HK && newM) {
        delete ex;
        ex = new cHuntKill(10);
    }

    if (mode != HT_BIN) {
        if (bBinary->logic(mX, mY, mouseState) == B_CLICK) {
            mode = HT_BIN;
            bBinary->setState(B_HOVER);

            delete ex;
            ex = new cBinary(10);

            exFinished = 0;
        }
    } else if (mode == HT_BIN && newM) {
        delete ex;
        ex = new cBinary(10);
    }

    if (mode != HT_PRMS) {
        if (bPrims->logic(mX, mY, mouseState) == B_CLICK) {
            mode = HT_PRMS;
            bPrims->setState(B_HOVER);

            delete ex;
            ex = new cPrims(10);

            exFinished = 0;
        }
    } else if (mode == HT_PRMS && newM) {
        delete ex;
        ex = new cPrims(10);
    }

    if (mode != HT_DSTR) {
        if (bDijkstra->logic(mX, mY, mouseState) == B_CLICK) {
            mode = HT_DSTR;
            bDijkstra->setState(B_HOVER);

            delete ex;
            ex = new cBinary(10);

            exFinished = 0;
        }
    } else if (mode == HT_DSTR && newM) {
        delete ex;
        ex = new cBinary(10);
    }

    return 0;
}

int gsHowTo::render() {
    SDL_FillRect(screen, NULL, SDL_MapRGB(screen->format, 255, 255, 255));

    applySurface(texts[mode - 1], screen, 0, 0);

    bBack->render();

    bMuurgroeier->render();
    bHuntKill->render();
    bBinary->render();
    bPrims->render();
    bDijkstra->render();

    applySurface(tChoose, screen, 10, 0);

    SDL_Rect tr;

    tr.x = 0;
    tr.y = 63;
    tr.w = SCR_W;
    tr.h = 8;
    SDL_FillRect(screen, &tr, 0);

    tr.y = 104;
    SDL_FillRect(screen, &tr, 0);

    ex->render(true);

    SDL_Flip(screen);

    return 0;
}

gsHowTo::gsHowTo() {
    bBack = new cButton("Terug", 10, 508, false, fNormal, B_VISIBLE, B_STANDARD);

    bMuurgroeier = new cButton("Muurgroeier", 10, 74, false, fTag, B_VISIBLE, B_STANDARD);
    bHuntKill = new cButton("Hunt&Kill", 145, 74, false, fTag, B_VISIBLE, B_STANDARD);
    bBinary = new cButton("Binair", 247, 74, false, fTag, B_VISIBLE, B_STANDARD);
    bPrims = new cButton("Prim's", 315, 74, false, fTag, B_VISIBLE, B_STANDARD);
    bDijkstra = new cButton("Dijkstra's", 390, 74, false, fTag, B_VISIBLE, B_STANDARD);

    tChoose = TTF_RenderText_Shaded(fHeader, "Kies een algoritme:", cBlack, cWhite);

    mode = HT_MG;
    bMuurgroeier->setState(B_NEUTRAL);
    ex = new cMG(10);

    prevMState = MAZE_MGNR;

    texts[0] = loadImage("Media/mg.png");
    texts[1] = loadImage("Media/hk.png");
    texts[2] = loadImage("Media/bin.png");
    texts[3] = loadImage("Media/prms.png");
    texts[4] = loadImage("Media/dstr.png");
}

gsHowTo::~gsHowTo() {
    delete bBack;

    delete bMuurgroeier;
    delete bHuntKill;
    delete bBinary;
    delete bPrims;
    delete bDijkstra;

    delete ex;

    SDL_FreeSurface(tChoose);

    for (int i = 0; i < 5; i++) {
        SDL_FreeSurface(texts[i]);
    }
}
