
#include "cMazeCell.h"
#include "cMazeField.h"
#include "constants.h"
#include "functions.h"
#include "gamestates.h"
#include "globals.h"

int gsMaze::logic() {
    if (bBack->logic(mX, mY, mouseState) == B_CLICK) {
        gameMan->setNextState(STATE_MENU);
    }

    speed->logic(mX, mY, mouseState);

    path->logic(mX, mY, mouseState);

    int mazeState = maze->logic(speed->getSelected(), path->getSelected());

    int which;
    waitBox.y = 20;
    waitBox.w = 8;
    waitBox.h = 8;

    if (mazeState == MAZE_MGNR) {
        if (speed->getSelected() == 0 || speed->getSelected() == 1) { // pauze
            indicMode = 1;
        } else {
            which = (SDL_GetTicks() - startFrame) / 400;
            which = which % 4;
            if (which == 3) {which = 1;}
            waitBox.x = 108 + which * (8 + 1);
            indicMode = 2;
        }
    } else if (mazeState == MAZE_PATH) {
        if (speed->getSelected() == 0 || speed->getSelected() == 1) { // pauze
            indicMode = 1;
        } else {
            which = (SDL_GetTicks() - startFrame) / 400;
            which = which % 4;
            if (which == 3) {which = 1;}
            waitBox.x = 108 + which * (8 + 1);
            indicMode = 2;
        }
    } else if (mazeState == MAZE_DONE) {
        indicMode = 3;
    }

    return 0;
}

int gsMaze::render() {
    SDL_FillRect(screen, NULL, SDL_MapRGB(screen->format, 255, 255, 255));

    applySurface(tagSpeed, screen, 310, 0);
    applySurface(tagPause, screen, 200, 22);
    applySurface(tagFast, screen, 450, 22);
    applySurface(tagPath, screen, 151, 0);

    bBack->render();

    speed->render();

    path->render();

    maze->render(path->getSelected());

    switch (indicMode) {
        case 1:
            waitBox.x = 110;
            waitBox.y = 13;
            waitBox.w = 6;
            waitBox.h = 25;
            SDL_FillRect(screen, &waitBox, SDL_MapRGB(screen->format, 0, 0, 0));

            waitBox.x = 122;
            waitBox.y = 13;
            waitBox.w = 6;
            waitBox.h = 25;
            SDL_FillRect(screen, &waitBox, SDL_MapRGB(screen->format, 0, 0, 0));
            break;
        case 2:
            SDL_FillRect(screen, &waitBox, SDL_MapRGB(screen->format, 0, 0, 0));
            break;
        case 3:
            applySurface(check, screen, 110, 13);
            break;
    }

    SDL_Flip(screen);

    return 0;
}

gsMaze::gsMaze() {
    bBack = new cButton("Terug", 10, 5, false, fNormal, B_VISIBLE, B_STANDARD);

    check = loadImage("Media/check.png");

    tagPause = TTF_RenderText_Shaded(fTag, "Pauze", cBlack, cWhite);
    tagFast = TTF_RenderText_Shaded(fTag, "Snel", cBlack, cWhite);
    tagSpeed = TTF_RenderText_Shaded(fTag, "Snelheid", cBlack, cWhite);
    tagPath = TTF_RenderText_Shaded(fTag, "Pad", cBlack, cWhite);

    speed = new cCheckboxes(260, 25, 21, true, MAXSPEED, 20, 2);
    path = new cCheckboxes(159, 25, 0, false, 1, 20, 2);

    int mazeSize;
    switch (gameMan->params[PARAM_SIZE]) {
        case 1:
            // maze = new cBinary(5);
            mazeSize = 5;
            break;
        case 2:
            // maze = new cBinary(20);
            mazeSize = 25;
            break;
        case 3:
            // maze = new cBinary(50);
            mazeSize = 50;
            break;
        default:
            // maze = new cBinary(50);
            mazeSize = 50;
            break;
    }

    switch (gameMan->params[PARAM_ALGO]) {
        case 1:
            maze = new cMG(mazeSize);
            break;
        case 2:
            maze = new cHuntKill(mazeSize);
            break;
        case 3:
            maze = new cBinary(mazeSize);
            break;
        case 4:
            maze = new cPrims(mazeSize);
            break;
    }

    switch (gameMan->params[PARAM_SPD]) {
        case 1:
            speed->setSelected(2);
            break;
        case 2:
            speed->setSelected(8);
            break;
        case 3:
            speed->setSelected(9);
            break;
    }

    startFrame = SDL_GetTicks();
}

gsMaze::~gsMaze() {
    delete bBack;
    delete maze;
    delete speed;

    SDL_FreeSurface(tagPause);
    SDL_FreeSurface(tagFast);
    SDL_FreeSurface(tagSpeed);
    SDL_FreeSurface(check);
}
